package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * A converter mainly designed for converting anime to web-playable mp4
 */
public class AnimeConverter extends AbstractConverter {

    private final String[] audioLanguages;
    private final String[] subtitleLanguages;
    private final boolean toStereo;
    private final boolean preventSignsAndSongs;
    private final int forcedAudioIndex;
    private final int forcedSubtitleIndex;

    /**
     * Instantiates a new anime converter
     *
     * @param ffprobePath          <p>Path/command to ffprobe.</p>
     * @param ffmpegPath           <p>Path/command to ffmpeg.</p>
     * @param audioLanguages       <p>List of wanted audio languages in descending order.</p>
     * @param subtitleLanguages    <p>List of wanted subtitle languages in descending order.</p>
     * @param toStereo             <p>Convert video with several audio channels to stereo.</p>
     * @param preventSignsAndSongs <p>Prevent subtitles only converting signs and songs (not speech).</p>
     * @param forcedAudioIndex     <p>A specific audio stream to force. 0-indexed from the first audio stream found</p>
     * @param forcedSubtitleIndex  <p>A specific subtitle stream to force. 0-indexed for the first subtitle stream found</p>
     */
    public AnimeConverter(String ffprobePath, String ffmpegPath, String[] audioLanguages, String[] subtitleLanguages,
                          boolean toStereo, boolean preventSignsAndSongs, int forcedAudioIndex, int forcedSubtitleIndex) {
        super("mp4");
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.audioLanguages = audioLanguages;
        this.subtitleLanguages = subtitleLanguages;
        this.toStereo = toStereo;
        this.preventSignsAndSongs = preventSignsAndSongs;
        this.forcedAudioIndex = forcedAudioIndex;
        this.forcedSubtitleIndex = forcedSubtitleIndex;
    }

    @Override
    public String[] generateConversionCommand(String executable, File file, List<StreamObject> streams, String outFile) {
        List<String> command = FFMpegHelper.getFFMpegWebVideoCommand(executable, file.getName());
        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        //Get the first audio stream in accordance with chosen languages
        List<AudioStream> audioStreams = filterAudioStreams(filterStreamsByType(streams, AudioStream.class), audioLanguages);
        AudioStream audioStream = getNthAudioSteam(new ArrayList<>(audioStreams), Math.max(forcedAudioIndex, 0));

        //Get the first subtitle stream in accordance with chosen languages and signs and songs prevention
        List<SubtitleStream> subtitleStreams = filterSubtitleStreams(filterStreamsByType(streams,
                SubtitleStream.class), subtitleLanguages, preventSignsAndSongs);
        SubtitleStream subtitleStream = getNthSubtitleStream(new ArrayList<>(subtitleStreams),
                Math.max(forcedSubtitleIndex, 0));

        //Get the first video stream
        VideoStream videoStream = getFirstVideoStream(streams);

        //Add streams to output file
        FFMpegHelper.addAudioStream(command, audioStream, toStereo);
        FFMpegHelper.addSubtitleAndVideoStream(command, subtitleStream, videoStream, file);

        command.add(outFile);
        return command.toArray(new String[0]);
    }

    @Override
    public String[] getValidFormats() {
        return videoFormats;
    }

}
